/*
 *  Copyright (C) 2021 Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <drivers/i2c.h>
#include <kernel/dpl/DebugP.h>
#include "ti_drivers_config.h"
#include "ti_drivers_open_close.h"
#include "ti_board_open_close.h"

//extern uint32_t Board_i2cIOExpanderAddr();
#define INPUT_PORT0_REG_ADDRESS   0x00 // Register address for Input Port 0
#define I2C_READ_LEN              (1U)
#define SLAVE_ADDR                0x22

static void i2c_read_error_handler(uint16_t sample, int32_t status);

void empty_main(void *arg0)
{
    uint16_t        sample;
    int32_t         status;
    uint8_t         txBuffer[1];
//    uint32_t        i2cReadSlaveAddr;
    uint8_t         rxBuffer[I2C_READ_LEN];
    I2C_Handle      i2cHandle;
    I2C_Transaction i2cTransaction;

//   i2cReadSlaveAddr     = Board_i2cIOExpanderAddr();
    i2cHandle = gI2cHandle[CONFIG_I2C0];
//    DebugP_log("i2cReadSlaveAddr: %u\r\n", i2cReadSlaveAddr);
    DebugP_log("[I2C] Read data ... !!!\r\n");

    /*Set the register address to read from*/
    txBuffer[0] = INPUT_PORT0_REG_ADDRESS;

    /* Set default transaction parameters */
    I2C_Transaction_init(&i2cTransaction);
    i2cTransaction.writeBuf      = txBuffer;
    i2cTransaction.writeCount    = 1;
    i2cTransaction.slaveAddress  = SLAVE_ADDR;

    /* Write the register address to the I/O expander*/
    status = I2C_transfer(i2cHandle, &i2cTransaction);
    if (status != I2C_STS_SUCCESS) {
        DebugP_logError("[I2C] Failed to set register address\r\n");
        return;
    }

    /* Set default transaction parameters for reading */
    i2cTransaction.readBuf      = rxBuffer;
    i2cTransaction.readCount    = I2C_READ_LEN;
    DebugP_log("i2cTransaction.readCount: %u\r\n", i2cTransaction.readCount);
    i2cTransaction.slaveAddress = SLAVE_ADDR;

    /* Read data from the I/O expander */
    for(sample = 0; sample < 20; sample++)
    {
    	rxBuffer[0]  = 0;
        status = I2C_transfer(i2cHandle, &i2cTransaction);
        if(status == I2C_STS_SUCCESS)
        {
            DebugP_log("[I2C] Sample %u: %u\r\n", sample, rxBuffer[0]);
        }
        else
        {
            i2c_read_error_handler(sample, status);
        }
    }

    DebugP_log("[I2C] Read data ... DONE !!!");
    if(status == SystemP_SUCCESS)
    {
        DebugP_log("All tests have passed!!\r\n");
    }
    else
    {
        DebugP_log("Some tests have failed!!\r\n");
    }

    return;
}

static void i2c_read_error_handler(uint16_t sample, int32_t status)
{
    switch(status)
    {
        case I2C_STS_ERR:
            DebugP_logError("[I2C] Sample %u: Generic error occurred", sample);
            break;
        case I2C_STS_ERR_TIMEOUT:
            DebugP_logError("[I2C] Sample %u: Timeout error occurred", sample);
            break;
        case I2C_STS_ERR_NO_ACK:
            DebugP_logError("[I2C] Sample %u: No acknowledgement received", sample);
            break;
        case I2C_STS_ERR_ARBITRATION_LOST:
            DebugP_logError("[I2C] Sample %u: Arbitration lost", sample);
            break;
        case I2C_STS_ERR_ACCESS_ERROR:
            DebugP_logError("[I2C] Sample %u: Bus Access error occurred", sample);
            break;
    }

    return;
}
